@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -98,7 +98,7 @@ assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
-             return get_assortativity_coefficient()
+             get_assortativity_coefficient()
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), a, a_err);
@@ -126,7 +126,7 @@ scalar_assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
         (gi,
          [&](auto&& graph, auto&& a2, auto&& a3)
          {
-             return get_scalar_assortativity_coefficient()
+             get_scalar_assortativity_coefficient()
                  (std::forward<decltype(graph)>(graph),
                   std::forward<decltype(a2)>(a2),
                   std::forward<decltype(a3)>(a3), a, a_err);
@@ -139,11 +139,14 @@ scalar_assortativity_coefficient(GraphInterface& gi, GraphInterface::deg_t deg,
 
 using namespace boost::python;
 
-void export_assortativity()
-{
-    def("assortativity_coefficient", &assortativity_coefficient);
-    def("scalar_assortativity_coefficient", &scalar_assortativity_coefficient);
+#define __MOD__ correlations
+#include "module_registry.hh"
+REGISTER_MOD
+([]
+ {
+     def("assortativity_coefficient", &assortativity_coefficient);
+     def("scalar_assortativity_coefficient", &scalar_assortativity_coefficient);
 
-    class_<empty_object>("empty_object");
-    class_<deleted_object>("deleted_object");
-}
+     class_<empty_object>("empty_object");
+     class_<deleted_object>("deleted_object");
+ });
