@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -40,6 +40,20 @@
 #include <cairomm/surface.h>
 #include PYCAIRO_HEADER
 
+#ifdef HAVE_CAIROMM_1_16
+using FontSlant = Cairo::ToyFontFace::Slant;
+using FontWeight = Cairo::ToyFontFace::Weight;
+#define FILL_RULE_EVEN_ODD Cairo::Context::FillRule::EVEN_ODD
+#define OPERATOR_SOURCE Cairo::Context::Operator::SOURCE
+#define OPERATOR_OVER Cairo::Context::Operator::OVER
+#else
+using FontSlant = Cairo::FontSlant;
+using FontWeight = Cairo::FontWeight;
+#define FILL_RULE_EVEN_ODD Cairo::FILL_RULE_EVEN_ODD
+#define OPERATOR_SOURCE Cairo::OPERATOR_SOURCE
+#define OPERATOR_OVER Cairo::OPERATOR_OVER
+#endif
+
 #include <boost/mpl/map/map50.hpp>
 
 #include <chrono>
@@ -634,8 +648,8 @@ public:
             if (text_pos == -1)
             {
                 cr.select_font_face(_attrs.template get<string>(VERTEX_FONT_FAMILY),
-                                    static_cast<Cairo::FontSlant>(_attrs.template get<int32_t>(VERTEX_FONT_SLANT)),
-                                    static_cast<Cairo::FontWeight>(_attrs.template get<int32_t>(VERTEX_FONT_WEIGHT)));
+                                    static_cast<FontSlant>(_attrs.template get<int32_t>(VERTEX_FONT_SLANT)),
+                                    static_cast<FontWeight>(_attrs.template get<int32_t>(VERTEX_FONT_WEIGHT)));
                 cr.set_font_size(get_user_dist(cr, _attrs.template get<double>(VERTEX_FONT_SIZE)));
                 Cairo::TextExtents extents;
                 cr.get_text_extents(text, extents);
@@ -968,8 +982,8 @@ public:
                 vector<double> text_offset;
 
                 cr.select_font_face(_attrs.template get<string>(VERTEX_FONT_FAMILY),
-                                    static_cast<Cairo::FontSlant>(_attrs.template get<int32_t>(VERTEX_FONT_SLANT)),
-                                    static_cast<Cairo::FontWeight>(_attrs.template get<int32_t>(VERTEX_FONT_WEIGHT)));
+                                    static_cast<FontSlant>(_attrs.template get<int32_t>(VERTEX_FONT_SLANT)),
+                                    static_cast<FontWeight>(_attrs.template get<int32_t>(VERTEX_FONT_WEIGHT)));
                 cr.set_font_size(get_user_dist(cr, _attrs.template get<double>(VERTEX_FONT_SIZE)));
                 text_pos = _attrs.template get<double>(VERTEX_TEXT_POSITION);
                 text_rotation = _attrs.template get<double>(VERTEX_TEXT_ROTATION);
@@ -1173,9 +1187,12 @@ public:
 
 
         bool sloppy = _attrs.template get<uint8_t>(EDGE_SLOPPY);
-        if (_s.get_size(cr) < get_user_dist(cr, res) &&
-            _t.get_size(cr) < get_user_dist(cr, res))
-            sloppy = true;
+        if (!sloppy)
+        {
+            if ((_s.get_size(cr) < get_user_dist(cr, res) &&
+                 _t.get_size(cr) < get_user_dist(cr, res)))
+                sloppy = true;
+        }
 
         bool seamless = _attrs.template get<uint8_t>(EDGE_SEAMLESS);
         if (marker_size < get_user_dist(cr, res) ||
@@ -1203,12 +1220,12 @@ public:
             _s.draw(cr, true);
             if (pos_begin != pos_end)
                 _t.draw(cr, true);
-            cr.set_fill_rule(Cairo::FILL_RULE_EVEN_ODD);
+            cr.set_fill_rule(FILL_RULE_EVEN_ODD);
             cr.clip();
 
             // seamlessly blend in separate surface
             cr.push_group();
-            cr.set_operator(Cairo::OPERATOR_SOURCE);
+            cr.set_operator(OPERATOR_SOURCE);
             draw_edge_markers(pos_begin_marker, pos_begin_d, pos_end_marker,
                               pos_end_d, controls, marker_size, cr);
             if (has_gradient)
@@ -1252,7 +1269,7 @@ public:
             vector<double> empty;
             cr.set_dash(empty, 0);
             cr.pop_group_to_source();
-            cr.set_operator(Cairo::OPERATOR_OVER);
+            cr.set_operator(OPERATOR_OVER);
             cr.reset_clip();
             if (!has_gradient)
                 cr.paint_with_alpha(get<3>(color));
@@ -1275,7 +1292,7 @@ public:
                 cr.rectangle(sx1, sy1, sx2 - sx1, sy2 - sy1);
                 draw_edge_markers(pos_begin_marker, pos_begin_d, pos_end_marker,
                                   pos_end_d, controls, marker_size, cr);
-                cr.set_fill_rule(Cairo::FILL_RULE_EVEN_ODD);
+                cr.set_fill_rule(FILL_RULE_EVEN_ODD);
                 cr.clip();
 
                 cr.rectangle(sx1, sy1, sx2 - sx1, sy2 - sy1);
@@ -1340,8 +1357,8 @@ public:
         {
             cr.save();
             cr.select_font_face(_attrs.template get<string>(EDGE_FONT_FAMILY),
-                                static_cast<Cairo::FontSlant>(_attrs.template get<int32_t>(EDGE_FONT_SLANT)),
-                                static_cast<Cairo::FontWeight>(_attrs.template get<int32_t>(EDGE_FONT_WEIGHT)));
+                                static_cast<FontSlant>(_attrs.template get<int32_t>(EDGE_FONT_SLANT)),
+                                static_cast<FontWeight>(_attrs.template get<int32_t>(EDGE_FONT_WEIGHT)));
             cr.set_font_size(get_user_dist(cr, _attrs.template get<double>(EDGE_FONT_SIZE)));
             double text_dist = _attrs.template get<double>(EDGE_TEXT_DISTANCE);
             text_dist = get_user_dist(cr, text_dist);
@@ -1840,7 +1857,7 @@ boost::python::object cairo_draw(GraphInterface& gi,
             populate_attrs<vertex_t, vertex_properties>(ovattrs, vattrs);
             populate_defaults(ovdefaults, vdefaults);
 
-            run_action<>()
+            run_action<>(false)
                 (gi,
                  [&](auto&& graph)
                  {
@@ -1869,7 +1886,7 @@ boost::python::object cairo_draw(GraphInterface& gi,
             Cairo::Context cr(PycairoContext_GET(ocr.ptr()));
 
             if (nodesfirst)
-                run_action<>()
+                run_action<>(false)
                     (gi,
                      [&](auto&& graph, auto&& a2, auto&& a3)
                      {
@@ -1883,7 +1900,7 @@ boost::python::object cairo_draw(GraphInterface& gi,
                      vertex_scalar_vector_properties(),
                      vorder_t())(pos, vorder);
 
-            run_action<>()
+            run_action<>(false)
                 (gi,
                  [&](auto&& graph, auto&& a2, auto&& a3)
                  {
@@ -1898,7 +1915,7 @@ boost::python::object cairo_draw(GraphInterface& gi,
 
             if (!nodesfirst)
             {
-                run_action<>()
+                run_action<>(false)
                     (gi,
                      [&](auto&& graph, auto&& a2, auto&& a3)
                      {
