@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -42,11 +42,13 @@ clique_iter_mh(GraphInterface& gi, boost::any ac, boost::any ax,
     multi_array_ref<int32_t, 1> Ed = get_array<int32_t, 1>(oEd);
 
     boost::python::object pret;
-    gt_dispatch<>()
+    gt_dispatch<>(false)
         ([&](auto& g){
+             GILRelease gil_release;
              auto ret = iter_mh(g, x.get_unchecked(), c.get_unchecked(),
                                 is_fac.get_unchecked(), is_max.get_unchecked(),
                                 Ed, N, E, beta, niter, rng);
+             gil_release.restore();
              pret = boost::python::make_tuple(get<0>(ret), get<1>(ret));
          },
          all_graph_views())
@@ -54,9 +56,12 @@ clique_iter_mh(GraphInterface& gi, boost::any ac, boost::any ax,
     return pret;
 };
 
-void export_clique_decomposition()
+#define __MOD__ inference
+#include "module_registry.hh"
+REGISTER_MOD
+([]
 {
     using namespace boost::python;
     def("clique_iter_mh", clique_iter_mh);
     def("clique_L_over", L_over);
-}
+});
