@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -55,8 +55,11 @@ python::object vi_multilevel_mcmc_sweep(python::object omcmc_state,
     return ret;
 }
 
-void export_vi_multilevel_mcmc()
+#define __MOD__ inference
+#include "module_registry.hh"
+REGISTER_MOD
+([]
 {
     using namespace boost::python;
     def("vi_multilevel_mcmc_sweep", &vi_multilevel_mcmc_sweep);
-}
+});
