@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -660,7 +660,7 @@ python::object do_get_all_shortest_paths(GraphInterface& gi, size_t s, size_t t,
 
     auto dispatch = [&](auto& yield)
         {
-            run_action<>()
+            run_action<>(false)
                 (gi, [&](auto& g, auto pred, auto weight)
                      {get_all_shortest_paths(gi, g, s, t, pred, weight, edges,
                                              yield);},
@@ -813,7 +813,7 @@ python::object do_get_all_paths(GraphInterface& gi, size_t s, size_t t,
     vprop_t visited = boost::any_cast<vprop_t>(avisited);
     auto dispatch = [&](auto& yield)
         {
-            run_action<>()
+            run_action<>(false)
                 (gi, [&](auto& g)
                 {
                     if (edges)
@@ -832,13 +832,16 @@ python::object do_get_all_paths(GraphInterface& gi, size_t s, size_t t,
 #endif // HAVE_BOOST_COROUTINE
 }
 
-void export_dists()
-{
-    python::def("get_dists", &get_dists);
-    python::def("get_all_preds", &do_get_all_preds);
-    python::def("get_all_shortest_paths", &do_get_all_shortest_paths);
-    python::def("get_all_paths", &do_get_all_paths);
+#define __MOD__ topology
+#include "module_registry.hh"
+REGISTER_MOD
+([]
+ {
+     python::def("get_dists", &get_dists);
+     python::def("get_all_preds", &do_get_all_preds);
+     python::def("get_all_shortest_paths", &do_get_all_shortest_paths);
+     python::def("get_all_paths", &do_get_all_paths);
 
-    python::def("get_weighted_succs", &get_weighted_succs);
-    python::def("get_random_shortest_path", &get_random_shortest_path);
-};
+     python::def("get_weighted_succs", &get_weighted_succs);
+     python::def("get_random_shortest_path", &get_random_shortest_path);
+ });
