@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -54,12 +54,14 @@ python::object similarity(GraphInterface& gi1, GraphInterface& gi2,
     if (weight2.empty())
         weight2 = ecmap_t();
     python::object s;
-    gt_dispatch<>()
+    gt_dispatch<>(false)
         ([&](const auto& g1, const auto& g2, auto ew1, auto l1)
          {
+             GILRelease gil_release;
              auto l2 = uncheck(l1, label2);
              auto ew2 = uncheck(ew1, weight2);
              auto ret = get_similarity(g1, g2, ew1, ew2, l1, l2, norm, asym);
+             gil_release.restore();
              s = python::object(ret);
          },
          all_graph_views(),
@@ -75,8 +77,11 @@ python::object similarity_fast(GraphInterface& gi1, GraphInterface& gi2,
                                boost::any label1, boost::any label2,
                                double norm, bool asym);
 
-void export_similarity()
-{
+#define __MOD__ topology
+#include "module_registry.hh"
+REGISTER_MOD
+([]
+ {
     python::def("similarity", &similarity);
     python::def("similarity_fast", &similarity_fast);
-};
+ });
