@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -18,14 +18,16 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from .. import Vector_size_t, group_vector_property
+from .. import Vector_size_t, group_vector_property, _parallel
 
 from . util import *
 
 import functools
 from abc import ABC, abstractmethod
-import gc
-
+import inspect
+import functools
+import textwrap
+import math
 import numpy
 
 import graph_tool.draw
@@ -60,7 +62,6 @@ def copy_state_wrap(func):
 
     return wrapper
 
-
 def mcmc_sweep_wrap(func):
     @functools.wraps(func)
     def wrapper(self, *args, **kwargs):
@@ -88,15 +89,115 @@ def mcmc_sweep_wrap(func):
         return ret
     return wrapper
 
-class MCMCState(ABC):
-    r"""Base state that implements single-flip MCMC sweeps"""
+def entropy_state_signature(cls):
+    def entropy(self, **kwargs):
+        return self._entropy(**dict(self.get_entropy_args(), **kwargs))
+    entropy.__signature__ = inspect.signature(cls._entropy)
+    intro, br, rest = cls._entropy.__doc__.partition("\n\n")
+    warn = f"""\
+    The default arguments of this function are overriden by those
+    obtained from :meth:`~{cls.__name__}.get_entropy_args`. To update the
+    defaults in a stateful way, :meth:`~{cls.__name__}.update_entropy_args`
+    should be called."""
+    warn = textwrap.dedent(warn)
+    warn = " ".join(warn.splitlines())
+    warn = textwrap.fill(warn, width=60, break_long_words=False,
+                         replace_whitespace=False)
+    warn = f".. warning::\n{warn}"
+    lines = cls._entropy.__doc__.splitlines()
+    m = min((len(l) - len(l.lstrip()) for l in lines[1:] if l.lstrip() != "")) \
+        if len(lines) > 1 else 0
+    warn = "\n".join([" " * (m if j == 0 else m + 4) + l.lstrip() for
+                      j, l in enumerate(warn.splitlines())])
+    entropy.__doc__ = intro + "\n\n" + warn
+    if len(rest) > 0:
+        entropy.__doc__ += "\n\n" + rest
+    setattr(cls, "entropy", entropy)
+    cls.get_entropy_args.__doc__ = \
+        cls.get_entropy_args.__doc__.replace("{entropy}",
+                                             f":meth:`~{cls.__name__}.entropy`")
+    cls.update_entropy_args.__doc__ = \
+        cls.update_entropy_args.__doc__.replace("{entropy}",
+                                                f":meth:`~{cls.__name__}.entropy`")
+    cls.reset_entropy_args.__doc__ = \
+        cls.reset_entropy_args.__doc__.replace("{entropy}",
+                                               f":meth:`~{cls.__name__}.entropy`")
+    return cls
+
+class EntropyState(ABC):
+    r"""Base state that implements entropy arguments."""
+
+    def __init__(self, entropy_args={}):
+        self._entropy_args = {}
+        self.reset_entropy_args()
+        self._entropy_args.update(entropy_args)
+
+    def get_entropy_args(self):
+        """Return the current default values for the parameters of the function
+        {entropy}, together with other operations that depend on them.
+        """
+        return self._entropy_args.copy()
+
+    def update_entropy_args(self, **kwargs):
+        """Update the default values for the parameters of the function
+        {entropy} from the keyword arguments, in a stateful way, together with
+        other operations that depend on them.
+
+        Values updated in this manner are preserved by the copying or pickling
+        of the state.
+        """
+        for k, v in kwargs.items():
+            if k not in self._entropy_args:
+                raise ValueError(f"unrecognized entropy argument: {k}")
+            self._entropy_args[k] = v
+
+    def reset_entropy_args(self):
+        """Reset the current default values for the parameters of the function
+        {entropy}, together with other operations that depend on them.
+        """
+        sig = inspect.signature(self._entropy)
+        for k, v in sig.parameters.items():
+            if k in ["self", "kwargs"]:
+                continue
+            self._entropy_args[k] = v.default
+
+    def __getstate__(self):
+        eargs = self.get_entropy_args()
+        sig = inspect.signature(self.entropy)
+        for k, v in sig.parameters.items():
+            if k in eargs and eargs[k] == v:
+                eargs.pop(k)
+        return dict(entropy_args=eargs)
 
     @abstractmethod
-    def _mcmc_sweep_dispatch(self, mcmc_state):
+    def _entropy(self, **kwargs):
         pass
 
     @abstractmethod
-    def _get_entropy_args(self, kwargs):
+    def _gen_eargs(self, args):
+        pass
+
+    def _get_entropy_args(self, eargs, consume=False):
+        if not consume:
+            eargs = dict(self._entropy_args, **eargs)
+        ea = self._gen_eargs(eargs)
+        for arg in list(eargs.keys()):
+            if arg not in self._entropy_args:
+                continue
+            setattr(ea, arg, eargs.pop(arg, self._entropy_args[arg]))
+        for k in ["self", "kwargs"]:
+            eargs.pop(k, None)
+        if len(eargs) > 0 and not consume:
+            raise ValueError("unrecognized entropy arguments: " +
+                             str(list(eargs.keys())))
+        return ea
+
+
+class MCMCState(EntropyState):
+    r"""Base state that implements single-flip MCMC sweeps."""
+
+    @abstractmethod
+    def _mcmc_sweep_dispatch(self, mcmc_state):
         pass
 
     @mcmc_sweep_wrap
@@ -186,17 +287,13 @@ class MCMCState(ABC):
         else:
             return mcmc_state
 
-class MultiflipMCMCState(ABC):
-    r"""Base state that implements multiflip (merge-split) MCMC sweeps"""
+class MultiflipMCMCState(EntropyState):
+    r"""Base state that implements multiflip (merge-split) MCMC sweeps."""
 
     @abstractmethod
     def _multiflip_mcmc_sweep_dispatch(self, mcmc_state):
         pass
 
-    @abstractmethod
-    def _get_entropy_args(self, kwargs):
-        pass
-
     @mcmc_sweep_wrap
     def multiflip_mcmc_sweep(self, beta=1., c=.5, psingle=None, psplit=1,
                              pmerge=1, pmergesplit=1, pmovelabel=0, d=0.01,
@@ -237,6 +334,9 @@ class MultiflipMCMCState(ABC):
         entropy_args : ``dict`` (optional, default: ``{}``)
             Entropy arguments, with the same meaning and defaults as in
             :meth:`graph_tool.inference.BlockState.entropy`.
+        accept_stats : ``dict`` (optional, default: ``None``)
+            If provided, this dictionary will be updated with the proposal and
+            acceptance counts for each kind of move.
         verbose : ``bool`` (optional, default: ``False``)
             If ``verbose == True``, detailed information will be displayed.
 
@@ -292,38 +392,36 @@ class MultiflipMCMCState(ABC):
 
         return dS, nattempts, nmoves
 
-class MultilevelMCMCState(ABC):
-    r"""Base state that implements multilevel agglomerative MCMC sweeps"""
+class MultilevelMCMCState(EntropyState):
+    r"""Base state that implements multilevel agglomerative MCMC sweeps."""
 
     @abstractmethod
     def _multilevel_mcmc_sweep_dispatch(self, mcmc_state):
         pass
 
-    @abstractmethod
-    def _get_entropy_args(self, kwargs):
-        pass
-
     def _get_bclabel(self):
         return None
 
     @mcmc_sweep_wrap
-    def multilevel_mcmc_sweep(self, niter=1, beta=1., c=.5, psingle=None,
-                              pmultilevel=1, d=0.01, r=0.9, random_bisect=True,
-                              merge_sweeps=10, mh_sweeps=10, init_r=0.99,
-                              init_beta=1., gibbs=False, B_min=1,
+    def multilevel_mcmc_sweep(self, niter=1, beta=numpy.inf, c=.5, d=0.01, r=0.9,
+                              random_bisect=True, merge_sweeps=10, mh_sweeps=10,
+                              init_r=0.99, init_min_iter=5, init_beta=1.,
+                              gibbs=False, B_min=1,
                               B_max=numpy.iinfo(numpy.uint64).max, b_min=None,
                               b_max=None, M=None, cache_states=True,
+                              force_accept=False, parallel=False,
                               entropy_args={}, verbose=False, **kwargs):
-        r"""Perform ``niter`` sweeps of a multilevel agglomerative acceptance-rejection
-        MCMC to sample network partitions, that uses a bisection search on the
-        number of groups, together with group merges and singe-node moves.
+        r"""Perform ``niter`` sweeps of a multilevel agglomerative
+        acceptance-rejection pseudo-MCMC (i.e. detailed balance is not
+        preserved) to sample network partitions, that uses a bisection search on
+        the number of groups, together with group merges and singe-node moves.
 
         Parameters
         ----------
         niter : ``int`` (optional, default: ``1``)
             Number of sweeps to perform. During each sweep, a move attempt is
             made for each node, on average.
-        beta : ``float`` (optional, default: ``1.``)
+        beta : ``float`` (optional, default: :const:`numpy.inf`)
             Inverse temperature.
         c : ``float`` (optional, default: ``.5``)
             Sampling parameter ``c`` for move proposals: For :math:`c\to 0` the
@@ -331,11 +429,6 @@ class MultilevelMCMCState(ABC):
             node and their block connections; for :math:`c\to\infty` the blocks
             are sampled randomly. Note that only for :math:`c > 0` the MCMC is
             guaranteed to be ergodic.
-        psingle : ``float`` (optional, default: ``None``)
-            Relative probability of proposing a single node move. If ``None``,
-            it will be selected as the number of nodes in the graph.
-        pmultilevel : ``float`` (optional, default: ``1``)
-            Relative probability of proposing a multilevel move.
         d : ``float`` (optional, default: ``.01``)
             Probability of selecting a new (i.e. empty) group for a given
             single-node move.
@@ -355,6 +448,8 @@ class MultilevelMCMCState(ABC):
             bisection search. A number of single-node Metropolis-Hastings sweeps
             is done until the number of groups is shrunk by a factor that is
             larger than this parameter.
+        init_min_iter : ``int`` (optional, default: ``5``)
+            Minimum number of iterations at the intialization phase.
         init_beta : ``float`` (optional, default: ``1.``)
             Inverse temperature to be used for the very first sweep of the
             initialization phase.
@@ -375,6 +470,12 @@ class MultilevelMCMCState(ABC):
         cache_states : ``bool`` (optional, default: ``True``)
             If ``True``, intermediary states will be cached during the bisection
             search.
+        force_accept : ``bool`` (optional, default: ``False``)
+            If ``True``, new state will be accepted even if it does not improve
+            the objective function.
+        parallel : ``bool`` (optional, default: ``False``)
+            If ``True``, the algorithm will run in parallel (if enabled during
+            compilation).
         entropy_args : ``dict`` (optional, default: ``{}``)
             Entropy arguments, with the same meaning and defaults as in
             :meth:`graph_tool.inference.BlockState.entropy`.
@@ -405,8 +506,6 @@ class MultilevelMCMCState(ABC):
 
         """
 
-        if psingle is None:
-            psingle = self.g.num_vertices()
         merge_sweeps = max((merge_sweeps, 1))
         if M is None:
             M = self.g.num_vertices()
@@ -435,17 +534,13 @@ class MultilevelMCMCState(ABC):
         else:
             return mcmc_state
 
-class GibbsMCMCState(ABC):
-    r"""Base state that implements single flip MCMC sweeps"""
+class GibbsMCMCState(EntropyState):
+    r"""Base state that implements single flip MCMC sweeps."""
 
     @abstractmethod
     def _gibbs_sweep_dispatch(self, gibbs_state):
         pass
 
-    @abstractmethod
-    def _get_entropy_args(self, kwargs):
-        pass
-
     @mcmc_sweep_wrap
     def gibbs_sweep(self, beta=1., niter=1, entropy_args={},
                     allow_new_group=True, sequential=True, deterministic=False,
@@ -513,8 +608,8 @@ class GibbsMCMCState(ABC):
 
         return dS, nattempts, nmoves
 
-class MulticanonicalMCMCState(ABC):
-    r"""Base state that implements multicanonical MCMC sweeps"""
+class MulticanonicalMCMCState(EntropyState):
+    r"""Base state that implements multicanonical MCMC sweeps."""
 
     @abstractmethod
     def _multicanonical_sweep_dispatch(self, multicanonical_state):
@@ -596,17 +691,13 @@ class MulticanonicalMCMCState(ABC):
 
         return S, nattempts, nmoves
 
-class ExhaustiveSweepState(ABC):
-    r"""Base state that implements exhaustive enumerative sweeps"""
+class ExhaustiveSweepState(EntropyState):
+    r"""Base state that implements exhaustive enumerative sweeps."""
 
     @abstractmethod
     def _exhaustive_sweep_dispatch(self, exhaustive_state):
         pass
 
-    @abstractmethod
-    def _get_entropy_args(self, kwargs):
-        pass
-
     def exhaustive_sweep(self, entropy_args={}, callback=None, density=None,
                          vertices=None, initial_partition=None, max_iter=None):
         r"""Perform an exhaustive loop over all possible network partitions.
@@ -695,7 +786,6 @@ class ExhaustiveSweepState(ABC):
         else:
             return b_min
 
-
 class DrawBlockState(ABC):
     r"""Base state that implements group-based drawing."""
 
