@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -22,13 +22,11 @@ from .. import _prop, Graph, libcore, _get_rng, PropertyMap, \
     VertexPropertyMap, Vector_size_t, Vector_double, group_vector_property, \
     perfect_prop_hash
 from .. generation import condensation_graph, random_rewire, generate_sbm, \
-    solve_sbm_fugacities, generate_maxent_sbm
-from .. stats import remove_parallel_edges, remove_self_loops
+    solve_sbm_fugacities, generate_maxent_sbm, remove_parallel_edges, \
+    remove_self_loops
 from .. spectral import adjacency
-import random
-from numpy import *
+import numpy as np
 import math
-import numpy
 import copy
 import collections.abc
 import itertools
@@ -89,8 +87,8 @@ def get_block_graph(g, B, b, vcount=None, ecount=None, rec=None, drec=None):
             cg.ep["drec%d" % i] = ae[0]
             del ae[0]
 
-    rs = numpy.setdiff1d(numpy.arange(B, dtype="int"), br.fa,
-                         assume_unique=True)
+    rs = np.setdiff1d(np.arange(B, dtype="int"), br.fa,
+                      assume_unique=True)
     if len(rs) > 0:
         cg.add_vertex(len(rs))
         br.fa[-len(rs):] = rs
@@ -114,10 +112,12 @@ _q_cache_max_n = 10000
 def init_q_cache(max_n=None):
     if max_n is None:
         max_n = _q_cache_max_n
-    libinference.init_q_cache(min(_q_cache_max_n, max_n))
+    libinference.init_q_cache(min((_q_cache_max_n, max_n)))
 
+@entropy_state_signature
 class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
-                 GibbsMCMCState, MulticanonicalMCMCState, ExhaustiveSweepState):
+                 GibbsMCMCState, MulticanonicalMCMCState, ExhaustiveSweepState,
+                 DrawBlockState):
     r"""The stochastic block model state of a given graph.
 
     Parameters
@@ -194,11 +194,17 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
     dense_bg : ``bool`` (optional, default: ``False``)
         If ``True`` a dense matrix is used for the block graph, otherwise a
         sparse matrix will be used.
+    entropy_args: ``dict`` (optional, default: ``{}``)
+        Override default arguments for :meth:`~BlockState.entropy()` method and
+        releated operations.
     """
 
     def __init__(self, g, b=None, B=None, eweight=None, vweight=None, recs=[],
                  rec_types=[], rec_params=[], clabel=None, pclabel=None,
-                 bfield=None, Bfield=None, deg_corr=True, dense_bg=False, **kwargs):
+                 bfield=None, Bfield=None, deg_corr=True, dense_bg=False,
+                 entropy_args={}, **kwargs):
+        EntropyState.__init__(self, entropy_args=entropy_args)
+
         kwargs = kwargs.copy()
 
         if eweight is None:
@@ -235,25 +241,27 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
         # ensure we have at most as many blocks as nodes
         if B is not None and b is None:
-            B = min(B, self.g.num_vertices())
+            B = min((B, self.g.num_vertices()))
 
         if b is None:
             # create a random partition into B blocks.
             if B is None:
                 raise ValueError("either 'b' or 'B' must be specified")
-            B = min(B, self.g.num_vertices())
-            ba = random.randint(0, B, self.g.num_vertices())
-            ba[:B] = arange(B)        # avoid empty blocks
+            B = min((B, self.g.num_vertices()))
+            ba = np.random.randint(0, B, self.g.num_vertices())
+            ba[:B] = np.arange(B)        # avoid empty blocks
             if B < self.g.num_vertices():
-                random.shuffle(ba)
+                np.random.shuffle(ba)
             b = g.new_vp("int")
             b.fa = ba
             self.b = b
         else:
             # if a partition is available, we will incorporate it.
-            if isinstance(b, numpy.ndarray):
+            if isinstance(b, np.ndarray):
                 self.b = g.new_vp("int")
-                self.b.fa = b
+                ba = np.zeros(len(self.b.fa), dtype="int")
+                ba[:min((len(ba), len(b)))] = b[:min((len(ba), len(b)))]
+                self.b.fa = ba
             else:
                 self.b = b = g.own_property(b.copy(value_type="int32_t"))
             if B is None:
@@ -353,8 +361,8 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
             for i in range(len(self.rec)):
                 idx = self.rec[i].a != 0
                 if not isinstance(self.eweight, libinference.unity_eprop_t):
-                    idx = numpy.logical_and(idx, self.eweight.a > 0)
-                if numpy.any(idx):
+                    idx = np.logical_and(idx, self.eweight.a > 0)
+                if np.any(idx):
                     self.epsilon[i] = abs(self.rec[i].a[idx]).min() / 10
 
         self.bfield = g.new_vp("vector<double>") if bfield is None else self.g.own_property(bfield)
@@ -371,17 +379,11 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         self._state = libinference.make_block_state(self)
         self.bg.properties.clear()
 
-        assert all(self.recdx.a >= 0), self.recdx.a
+        if _bm_test():
+            assert all(self.recdx.a >= 0), self.recdx.a
 
         if deg_corr:
-            init_q_cache(max(2 * max(self.get_E(), self.get_N()), 100))
-
-        self._entropy_args = dict(adjacency=True, deg_entropy=True, dl=True,
-                                  partition_dl=True, degree_dl=True,
-                                  degree_dl_kind="distributed", edges_dl=True,
-                                  dense=False, multigraph=True, exact=True,
-                                  recs=True, recs_dl=True, beta_dl=1.,
-                                  Bfield=True)
+            init_q_cache(max((2 * max((self.get_E(), self.get_N())), 100)))
 
         if len(kwargs) > 0:
             warnings.warn("unrecognized keyword arguments: " +
@@ -426,8 +428,8 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                     defaults = {"alpha": 1,
                                 "beta": self.rec[i].fa.mean()}
                 else:
-                    defaults = {"alpha": numpy.nan,
-                                "beta": numpy.nan}
+                    defaults = {"alpha": np.nan,
+                                "beta": np.nan}
             elif rt == libinference.rec_type.real_normal:
                 if rec_params[i] != "microcanonical":
                     defaults = {"m0": self.rec[i].fa.mean(),
@@ -435,17 +437,17 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                                 "v0": self.rec[i].fa.std() ** 2,
                                 "nu0": 3}
                 else:
-                    defaults = {"m0": numpy.nan,
-                                "k0": numpy.nan,
-                                "v0": numpy.nan,
-                                "nu0": numpy.nan}
+                    defaults = {"m0": np.nan,
+                                "k0": np.nan,
+                                "v0": np.nan,
+                                "nu0": np.nan}
             elif rt == libinference.rec_type.discrete_geometric:
                 if rec_params[i] != "microcanonical":
                     defaults = {"alpha": 1,
                                 "beta": 1}
                 else:
-                    defaults = {"alpha": numpy.nan,
-                                "beta": numpy.nan}
+                    defaults = {"alpha": np.nan,
+                                "beta": np.nan}
             elif rt == libinference.rec_type.discrete_binomial:
                 if rec_params[i] != "microcanonical":
                     defaults = {"N": self.rec[i].fa.max(),
@@ -453,8 +455,8 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                                 "beta": 1}
                 else:
                     defaults = {"N": self.rec[i].fa.max(),
-                                "alpha": numpy.nan,
-                                "beta": numpy.nan}
+                                "alpha": np.nan,
+                                "beta": np.nan}
 
             ks = list(defaults.keys())
             if rec_params[i] != "microcanonical":
@@ -496,10 +498,6 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
     def __copy__(self):
         return self.copy()
 
-    def __deepcopy__(self, memo):
-        g = copy.deepcopy(self.g, memo)
-        return self.copy(g=g)
-
     def copy(self, g=None, eweight=None, vweight=None, b=None, B=None,
              deg_corr=None, clabel=None, overlap=False, pclabel=None,
              bfield=None, dense_bg=None, **kwargs):
@@ -553,7 +551,9 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
 
     def __getstate__(self):
-        state = dict(g=self.g,
+        state = EntropyState.__getstate__(self)
+        state = dict(state,
+                     g=self.g,
                      eweight=self.eweight if self.is_weighted else None,
                      vweight=self.vweight if self.is_weighted else None,
                      b=self.b,
@@ -573,7 +573,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         self.__init__(**state)
 
     def get_block_state(self, b=None, vweight=False, **kwargs):
-        r"""Returns a :class:`~graph_tool.inference.blockmodel.BlockState` corresponding
+        r"""Returns a :class:`~graph_tool.inference.BlockState` corresponding
         to the block graph (i.e. the blocks of the current state become the
         nodes). The parameters have the same meaning as the in the
         constructor. If ``vweight == True`` the nodes of the block state are
@@ -634,7 +634,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                     recs.append(bg.new_ep("double", eweight.fa > 0))
                     rec_types.append(rt)
                     rec_params.append("microcanonical")
-                elif numpy.isnan(rp.a).sum() == 0:
+                elif np.isnan(rp.a).sum() == 0:
                     continue
                 elif rt in [libinference.rec_type.discrete_geometric,
                             libinference.rec_type.discrete_binomial,
@@ -700,21 +700,21 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         :math:`H=-\sum_r\frac{n_r}{N}\ln \frac{n_r}{N}`, where :math:`n_r` is
         the number of nodes in group r.
         """
-        w = numpy.array(self.wr.a, dtype="double")
+        w = np.array(self.wr.a, dtype="double")
         w = w[w>0]
         w /= w.sum()
-        return numpy.exp(-(w*log(w)).sum())
+        return np.exp(-(w*np.log(w)).sum())
 
     def get_bclabel(self, clabel=None):
         r"""Returns a :class:`~graph_tool.VertexPropertyMap` corresponding to constraint
         labels for the block graph."""
 
         bclabel = self.bg.new_vertex_property("int")
-        idx = self.vweight.a > 0
-        reverse_map(self.b.a[idx], bclabel)
+        idx = self.vweight.fa > 0
+        reverse_map(self.b.fa[idx], bclabel)
         if clabel is None:
             clabel = self.clabel
-        pmap(bclabel, clabel.a[idx])
+        pmap(bclabel, clabel.fa[idx])
         return bclabel
 
     def _set_bclabel(self, bstate):
@@ -759,7 +759,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         return self.b
 
     def get_state(self):
-        """Alias to :meth:`~graph_tool.inference.blockmodel.BlockState.get_blocks`."""
+        """Alias to :meth:`~graph_tool.inference.BlockState.get_blocks`."""
         return self.get_blocks()
 
     def set_state(self, b):
@@ -796,12 +796,12 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         return self.wr
 
     @copy_state_wrap
-    def entropy(self, adjacency=True, dl=True, partition_dl=True,
-                degree_dl=True, degree_dl_kind="distributed", edges_dl=True,
-                dense=False, multigraph=True, deg_entropy=True, recs=True,
-                recs_dl=True, beta_dl=1., Bfield=True, exact=True, **kwargs):
-        r"""Calculate the entropy (a.k.a. negative log-likelihood) associated
-        with the current block partition.
+    def _entropy(self, adjacency=True, dl=True, partition_dl=True,
+                 degree_dl=True, degree_dl_kind="distributed", edges_dl=True,
+                 dense=False, multigraph=True, deg_entropy=True, recs=True,
+                 recs_dl=True, beta_dl=1., Bfield=True, exact=True, **kwargs):
+        r"""Calculate the description length (a.k.a. negative joint
+        log-likelihood) associated with the current block partition.
 
         Parameters
         ----------
@@ -838,6 +838,9 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
             length will be included.
         beta_dl : ``double`` (optional, default: ``1.``)
             Prior inverse temperature.
+        Bfield : ``bool`` (optional, default: ``False``)
+            If True, the ``Bfield`` parameter passed to the construtor will be
+            taken into account.
         exact : ``bool`` (optional, default: ``True``)
             If ``True``, the exact expressions will be used. Otherwise,
             Stirling's factorial approximation will be used for some terms.
@@ -976,13 +979,21 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                 P(\boldsymbol{k}|\boldsymbol{e},\boldsymbol{b}) = \prod_r\frac{\prod_k\eta_k^r!}{n_r!} \prod_r q(e_r, n_r)^{-1}
 
             with :math:`\eta_k^r` being the number of nodes with degree
-            :math:`k` in group :math:`r`, and :math:`q(n,m)` being the number of
+            :math:`k` in group :math:`r`, and :math:`q(m,n)` being the number of
             `partitions
             <https://en.wikipedia.org/wiki/Partition_(number_theory)>`_ of
-            integer :math:`n` into at most :math:`m` parts.
+            integer :math:`m` into at most :math:`n` parts. This is given by the
+            recurrence
+
+            .. math::
+
+                q(m, n) = q(m, n-1) + q(m-n, n),
+
+            with boundary conditions :math:`q(m,n) = q(m,m)` for :math:`n>m`,
+            :math:`q(m,0) = 0` for :math:`m>0`, and :math:`q(0,0) = 1`.
 
             This corresponds to a prior for the degree sequence conditioned on
-            the degree frequencies, which are themselves sampled from a uniform
+            the degree counts, which are themselves sampled from a uniform
             hyperprior. This option should be preferred in most cases.
 
         3. ``degree_dl_kind == "entropy"``
@@ -1018,7 +1029,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
         """
 
-        eargs = self._get_entropy_args(locals(), ignore=["self", "kwargs"])
+        eargs = self._get_entropy_args(locals())
 
         S = self._state.entropy(eargs, kwargs.pop("propagate", False))
 
@@ -1028,57 +1039,28 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                              str(list(kwargs.keys())))
         return S
 
-    def _get_entropy_args(self, kwargs, ignore=None):
-        kwargs = dict(self._entropy_args, **kwargs)
-        if ignore is not None:
-            for a in ignore:
-                kwargs.pop(a, None)
-        args = DictState(kwargs)
-        deg_dl_kind = args.degree_dl_kind
-        del kwargs["degree_dl_kind"]
+    def _gen_eargs(self, args):
+        return libinference.entropy_args()
+
+    def _get_entropy_args(self, kwargs, consume=False):
+        if not consume:
+            kwargs = kwargs.copy()
+        deg_dl_kind = kwargs.get("degree_dl_kind",
+                                 self._entropy_args["degree_dl_kind"])
         if deg_dl_kind == "entropy":
             kind = libinference.deg_dl_kind.ent
         elif deg_dl_kind == "uniform":
             kind = libinference.deg_dl_kind.uniform
         elif deg_dl_kind == "distributed":
             kind = libinference.deg_dl_kind.dist
-        ea = libinference.entropy_args()
-        ea.exact = args.exact
-        ea.dense = args.dense
-        ea.multigraph = args.multigraph
-        ea.adjacency = args.adjacency
-        ea.deg_entropy = args.deg_entropy
-        ea.recs = args.recs
-        del kwargs["exact"]
-        del kwargs["dense"]
-        del kwargs["multigraph"]
-        del kwargs["adjacency"]
-        del kwargs["deg_entropy"]
-        del kwargs["recs"]
-        if args.dl:
-            ea.partition_dl = args.partition_dl
-            ea.degree_dl = args.degree_dl
-            ea.edges_dl = args.edges_dl
-            ea.recs_dl = args.recs_dl
-        else:
+        kwargs["degree_dl_kind"] = kind
+        dl = kwargs.get("dl", self._entropy_args["dl"])
+        ea = super()._get_entropy_args(kwargs, consume)
+        if not dl:
             ea.partition_dl = False
             ea.degree_dl = False
             ea.edges_dl = False
             ea.recs_dl = False
-        ea.degree_dl_kind = kind
-        ea.beta_dl = args.beta_dl
-        ea.Bfield = args.Bfield
-        del kwargs["dl"]
-        del kwargs["partition_dl"]
-        del kwargs["degree_dl"]
-        del kwargs["edges_dl"]
-        del kwargs["recs_dl"]
-        del kwargs["beta_dl"]
-        del kwargs["Bfield"]
-        kwargs.pop("callback", None)
-        if len(kwargs) > 0:
-            raise ValueError("unrecognized entropy arguments: " +
-                             str(list(kwargs.keys())))
         return ea
 
     def get_matrix(self):
@@ -1125,7 +1107,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
     def virtual_vertex_move(self, v, s, **kwargs):
         r"""Computes the entropy difference if vertex ``v`` is moved to block ``s``. The
         remaining parameters are the same as in
-        :meth:`graph_tool.inference.blockmodel.BlockState.entropy`."""
+        :meth:`graph_tool.inference.BlockState.entropy`."""
         return self._state.virtual_move(int(v), self.b[v], s,
                                         self._get_entropy_args(dict(self._entropy_args,
                                                                     **kwargs)))
@@ -1139,8 +1121,8 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         if not isinstance(v, collections.abc.Iterable):
             self._state.move_vertex(int(v), s)
         else:
-            self._state.move_vertices(numpy.asarray(v, dtype="uint64"),
-                                      numpy.asarray(s, dtype="uint64"))
+            self._state.move_vertices(np.asarray(v, dtype="uint64"),
+                                      np.asarray(s, dtype="uint64"))
 
     def remove_vertex(self, v):
         r"""Remove vertex ``v`` from its current group.
@@ -1154,9 +1136,9 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
            twice.
         """
         if isinstance(v, collections.abc.Iterable):
-            if not isinstance(v, numpy.ndarray):
+            if not isinstance(v, np.ndarray):
                 v = list(v)
-            self._state.remove_vertices(numpy.asarray(v, dtype="uint64"))
+            self._state.remove_vertices(np.asarray(v, dtype="uint64"))
         else:
             self._state.remove_vertex(int(v))
 
@@ -1171,12 +1153,12 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
            added twice to the same group.
         """
         if isinstance(v, collections.abc.Iterable):
-            if not isinstance(v, numpy.ndarray):
+            if not isinstance(v, np.ndarray):
                 v = list(v)
-            if not isinstance(r, numpy.ndarray):
+            if not isinstance(r, np.ndarray):
                 r = list(r)
-            self._state.add_vertices(numpy.asarray(v, dtype="uint64"),
-                                     numpy.asarray(r, dtype="uint64"))
+            self._state.add_vertices(np.asarray(v, dtype="uint64"),
+                                     np.asarray(r, dtype="uint64"))
         else:
             self._state.add_vertex(int(v), r)
 
@@ -1193,7 +1175,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
     def get_move_prob(self, v, s, c=1., d=.1, reverse=False):
         r"""Compute the log-probability of a move proposal for vertex ``v`` to block ``s``
         according to sampling parameters ``c`` and ``d``, as obtained with
-        :meth:`graph_tool.inference.blockmodel.BlockState.sample_vertex_move`. If ``reverse
+        :meth:`graph_tool.inference.BlockState.sample_vertex_move`. If ``reverse
         == True``, the reverse probability of moving the node back from block
         ``s`` to its current one is obtained.
         """
@@ -1218,7 +1200,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         (with missing edges added and spurious edges deleted).
 
         The values in ``entropy_args`` are passed to
-        :meth:`~graph_tool.inference.blockmodel.BlockState.entropy()` to
+        :meth:`~graph_tool.inference.BlockState.entropy()` to
         calculate the log-probability.
 
         """
@@ -1374,7 +1356,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         the endpoints of each node have been assigned to a given block pair.
 
         This should be called multiple times, e.g. after repeated runs of the
-        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` function.
+        :meth:`graph_tool.inference.BlockState.mcmc_sweep` function.
 
         Parameters
         ----------
@@ -1405,7 +1387,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
            ...     ret = state.mcmc_sweep(niter=10)
            ...     pe = state.collect_edge_marginals(pe)
            >>> gt.bethe_entropy(g, pe)[0]
-           0.571383...
+           -20.424228...
 
         """
 
@@ -1425,7 +1407,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         node was assigned to a given block.
 
         This should be called multiple times, e.g. after repeated runs of the
-        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` function.
+        :meth:`graph_tool.inference.BlockState.mcmc_sweep` function.
 
         Parameters
         ----------
@@ -1463,7 +1445,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
            ...     ret = state.mcmc_sweep(niter=10)
            ...     pv = state.collect_vertex_marginals(pv)
            >>> gt.mf_entropy(g, pv)
-           15.037062...
+           33.560691...
            >>> gt.graph_draw(g, pos=g.vp["pos"], vertex_shape="pie",
            ...               vertex_pie_fractions=pv, output="polbooks_blocks_soft_B4.svg")
            <...>
@@ -1491,11 +1473,11 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
         r"""Collect a histogram of partitions.
 
         This should be called multiple times, e.g. after repeated runs of the
-        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` function.
+        :meth:`graph_tool.inference.BlockState.mcmc_sweep` function.
 
         Parameters
         ----------
-        h : :class:`~graph_tool.inference.blockmodel.PartitionHist` (optional, default: ``None``)
+        h : :class:`~graph_tool.inference.PartitionHist` (optional, default: ``None``)
             Partition histogram. If not provided, an empty histogram will be created.
         update : float (optional, default: ``1``)
             Each call increases the current count by the amount given by this
@@ -1506,7 +1488,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
 
         Returns
         -------
-        h : :class:`~graph_tool.inference.blockmodel.PartitionHist` (optional, default: ``None``)
+        h : :class:`~graph_tool.inference.PartitionHist` (optional, default: ``None``)
             Updated Partition histogram.
 
         Examples
@@ -1524,7 +1506,7 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
            ...     ret = state.mcmc_sweep(niter=10)
            ...     ph = state.collect_partition_histogram(ph)
            >>> gt.microstate_entropy(ph)
-           126.171367...
+           137.247829...
 
         """
 
@@ -1534,22 +1516,6 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                                         h, update, unlabel)
         return h
 
-    def draw(self, **kwargs):
-        r"""Convenience wrapper to :func:`~graph_tool.draw.graph_draw` that
-        draws the state of the graph as colors on the vertices and edges."""
-        gradient = self.g.new_ep("double")
-        gradient = group_vector_property([gradient])
-        from graph_tool.draw import graph_draw
-        return graph_draw(self.g,
-                          vertex_fill_color=kwargs.get("vertex_fill_color",
-                                                       self.b),
-                          vertex_color=kwargs.get("vertex_color", self.b),
-                          edge_gradient=kwargs.get("edge_gradient",
-                                                   gradient),
-                          **dmask(kwargs, ["vertex_fill_color",
-                                           "vertex_color",
-                                           "edge_gradient"]))
-
     def sample_graph(self, canonical=False, multigraph=True, self_loops=True,
                      sample_params=False, max_ent=False, n_iter=1000):
         r"""Sample a new graph from the fitted model.
@@ -1630,12 +1596,12 @@ class BlockState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                 if not self.g.is_directed():
                     p = 2 * self.g.num_edges() / ((B + 1) * B)
                 idx = probs.nonzero()
-                probs[idx] = numpy.random.gamma(probs[idx] + 1, p/(p + 1))
+                probs[idx] = np.random.gamma(probs[idx] + 1, p/(p + 1))
                 for r in rs:
                     idx = self.b.fa == r
-                    out_degs[idx] = numpy.random.dirichlet(out_degs[idx] + 1)
+                    out_degs[idx] = np.random.dirichlet(out_degs[idx] + 1)
                     if in_degs is not None:
-                        in_degs[idx] = numpy.random.dirichlet(in_degs[idx] + 1)
+                        in_degs[idx] = np.random.dirichlet(in_degs[idx] + 1)
 
             g = generate_sbm(b=self.b.fa, probs=probs,
                              in_degs=in_degs, out_degs=out_degs,
@@ -1696,7 +1662,7 @@ def bethe_entropy(g, p):
         Vertex property map with vector-type values, storing the accumulated
         block membership counts. These are the node marginals, as would be
         returned by the
-        :meth:`~graph_tool.inference.blockmodel.BlockState.collect_vertex_marginals`
+        :meth:`~graph_tool.inference.BlockState.collect_vertex_marginals`
         method.
 
     Notes
@@ -1773,7 +1739,7 @@ def microstate_entropy(h, unlabel=True):
 
     Parameters
     ----------
-    h : :class:`~graph_tool.inference.blockmodel.PartitionHist` (optional, default: ``None``)
+    h : :class:`~graph_tool.inference.PartitionHist` (optional, default: ``None``)
         Partition histogram.
     unlabel : bool (optional, default: ``True``)
         If ``True``, a canonical labelling of the groups will be used, so that
