@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -27,8 +27,7 @@ from . blockmodel import *
 from . overlap_blockmodel import *
 from . layered_blockmodel import *
 
-from numpy import *
-import numpy
+import numpy as np
 import copy
 
 class NestedBlockState(object):
@@ -93,7 +92,8 @@ class NestedBlockState(object):
                                   recs_dl=False,
                                   beta_dl=1.)
 
-        self.levels = [base_type(g, b=bs[0] if bs is not None else None,
+        g_ = self.state_args.pop("g", g) # for OverlapBlockState
+        self.levels = [base_type(g_, b=bs[0] if bs is not None else None,
                                  **self.state_args)]
 
         if bs is None:
@@ -101,7 +101,7 @@ class NestedBlockState(object):
                 N = 2 * self.levels[0].get_N()
             else:
                 N = self.levels[0].get_N()
-            L = int(numpy.ceil(numpy.log2(N)))
+            L = int(np.ceil(np.log2(N)))
             bs = [None] * (L + 1)
 
         for i, b in enumerate(bs[1:]):
@@ -199,7 +199,8 @@ class NestedBlockState(object):
         base_state.pop("epsilon", None)
         base_state.pop("drec", None)
         state_args = dict(self.state_args, **base_state)
-        state_args.pop("g", None)
+        if self.base_type is not OverlapBlockState:
+            state_args.pop("g", None)
         state_args.pop("b", None)
         state = dict(g=self.g, bs=self.get_bs(),
                      base_type=type(self.levels[0]),
@@ -215,7 +216,7 @@ class NestedBlockState(object):
         """Get hierarchy levels as a list of :class:`numpy.ndarray` objects with the
         group memberships at each level.
         """
-        return [s.b.fa.copy() for s in self.levels]
+        return [s.b.copy() if l == 0 else s.b.fa.copy() for l, s in enumerate(self.levels)]
 
     def get_state(self):
         """Alias to :meth:`~NestedBlockState.get_bs`."""
@@ -319,7 +320,7 @@ class NestedBlockState(object):
                 for s in self.levels:
                     B_E_D = s._state.get_B_E_D()
                     if B_E_D > 0:
-                        S_D -= log(B_E_D)
+                        S_D -= np.log(B_E_D)
 
             S = 0
             for i in range(len(self.levels[0].rec)):
@@ -327,7 +328,7 @@ class NestedBlockState(object):
                     continue
                 assert not _bm_test() or Lrecdx[i+1] >= 0, (i, Lrecdx[i+1])
                 S += -libinference.positive_w_log_P(Lrecdx[0], Lrecdx[i+1],
-                                                    numpy.nan, numpy.nan,
+                                                    np.nan, np.nan,
                                                     self.levels[0].epsilon[i])
                 S += S_D
             return S
@@ -339,7 +340,7 @@ class NestedBlockState(object):
                     for j in range(self.levels[0].C):
                         B_E_D = s._state.get_layer(j).get_B_E_D()
                         if B_E_D > 0:
-                            S_D[j] -= log(B_E_D)
+                            S_D[j] -= np.log(B_E_D)
 
             S = 0
             for i in range(len(self.levels[0].rec)):
@@ -349,20 +350,23 @@ class NestedBlockState(object):
                     assert not _bm_test() or Lrecdx[j+1][i+1] >= 0, (i, j, Lrecdx[j+1][i+1])
                     S += -libinference.positive_w_log_P(Lrecdx[j+1][0],
                                                         Lrecdx[j+1][i+1],
-                                                        numpy.nan, numpy.nan,
+                                                        np.nan, np.nan,
                                                         self.levels[0].epsilon[i])
                     S += S_D[j]
             return S
 
     @copy_state_wrap
     def entropy(self, **kwargs):
-        """Compute the entropy of whole hierarchy.
+        """Obtain the description length (i.e. negative joint log-likelihood)
+        for the hierarchical partition.
 
         The keyword arguments are passed to the ``entropy()`` method of the
         underlying state objects
         (e.g. :class:`graph_tool.inference.BlockState.entropy`,
         :class:`graph_tool.inference.OverlapBlockState.entropy`, or
-        :class:`graph_tool.inference.LayeredBlockState.entropy`).  """
+        :class:`graph_tool.inference.LayeredBlockState.entropy`).
+
+        """
         S = 0
         for l in range(len(self.levels)):
             S += self.level_entropy(l, **dict(kwargs, test=False))
@@ -526,7 +530,7 @@ class NestedBlockState(object):
 
         lrange = list(kwargs.pop("ls", range(len(self.levels))))
         if kwargs.pop("ls_shuffle", True):
-            numpy.random.shuffle(lrange)
+            np.random.shuffle(lrange)
         for l in lrange:
             if check_verbose(verbose):
                 print(verbose_pad(verbose) + "level:", l)
@@ -699,8 +703,6 @@ class NestedBlockState(object):
 
         """
 
-        kwargs["psingle"] = kwargs.get("psingle", self.g.num_vertices())
-
         c = kwargs.pop("c", 1)
         if not isinstance(c, collections.abc.Iterable):
             c = [c * 2 ** l for l in range(0, len(self.levels))]
@@ -789,7 +791,10 @@ class NestedBlockState(object):
         r"""Convenience wrapper to :func:`~graph_tool.draw.draw_hierarchy` that
         draws the hierarchical state."""
         import graph_tool.draw
-        return graph_tool.draw.draw_hierarchy(self, **kwargs)
+        return graph_tool.draw.draw_hierarchy(self,
+                                              vcmap=kwargs.pop("vcmap",
+                                                               graph_tool.draw.default_cm),
+                                              **kwargs)
 
 def get_hierarchy_tree(state, empty_branches=False):
     r"""Obtain the nested hierarchical levels as a tree.
@@ -855,7 +860,7 @@ def get_hierarchy_tree(state, empty_branches=False):
             t_vertices.extend(t.add_vertex(s.num_vertices()))
         else:
             t_vertices.append(t.add_vertex(s.num_vertices()))
-        label.a[-s.num_vertices():] = arange(s.num_vertices())
+        label.a[-s.num_vertices():] = np.arange(s.num_vertices())
 
         # relative ordering based on total degree
         count = s.ep["count"].copy("double")
