@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -18,7 +18,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from .. import Graph, _get_rng, Vector_int32_t, Vector_size_t
+from .. import Graph, PropertyMap, _get_rng, Vector_int32_t, Vector_size_t, \
+    _parallel
 from . blockmodel import DictState
 from . base_states import *
 
@@ -27,6 +28,7 @@ dl_import("from . import libgraph_tool_inference as libinference")
 
 import numpy as np
 import math
+from functools import wraps
 
 class PartitionModeState(object):
     r"""The random label model state for a set of labelled partitions, which attempts
@@ -46,7 +48,7 @@ class PartitionModeState(object):
 
         If ``True``, the label alignment will be iterated until convergence upon
         initialization (otherwise
-        :meth:`~graph_tool.inference.partition_modes.PartitionModeState.replace_partitions`
+        :meth:`~graph_tool.inference.PartitionModeState.replace_partitions`
         needs to be called repeatedly).
 
     References
@@ -96,10 +98,14 @@ class PartitionModeState(object):
     def add_partition(self, b, relabel=True):
         r"""Adds partition ``b`` to the ensemble, after relabelling it if
         ``relabel=True``, and returns its index in the population."""
+        def c(x):
+            if isinstance(x, PropertyMap):
+                return x.fa
+            return x
         if self.nested:
-            b = [Vector_int32_t(init=x) for x in b]
+            b = [Vector_int32_t(init=c(x)) for x in b]
         else:
-            b = [Vector_int32_t(init=b)]
+            b = [Vector_int32_t(init=c(b))]
         i = self._base.add_partition(b, relabel)
         self.bs[i] = b
         return i
@@ -131,7 +137,7 @@ class PartitionModeState(object):
         return self._base.virtual_add_partition(b, relabel)
 
     def replace_partitions(self):
-        r"""Removes and re-adds every partition, after relabelling, an returns
+        r"""Removes and re-adds every partition, after relabelling, and returns
         the entropy difference (negative log probability)."""
         return self._base.replace_partitions(_get_rng())
 
@@ -151,7 +157,7 @@ class PartitionModeState(object):
     def align_mode(self, mode):
         r"""Relabel entire ensemble to align with another ensemble given by ``mode``,
         which should be an instance of
-        :class:`~graph_tool.inference.partition_modes.PartitionModeState`."""
+        :class:`~graph_tool.inference.PartitionModeState`."""
         self._base.align_mode(mode._base)
 
     def get_partition(self, i):
@@ -175,8 +181,8 @@ class PartitionModeState(object):
         return self._base.relabel()
 
     @copy_state_wrap
-    def entropy(self):
-        r"""Return the model entropy (negative log-likelihood)."""
+    def _entropy(self):
+        r"""Return the description length (negative joint log-likelihood)."""
         return self._base.entropy()
 
     def posterior_entropy(self, MLE=True):
@@ -205,7 +211,7 @@ class PartitionModeState(object):
         return self._base.posterior_lprob(b, MLE)
 
     def get_coupled_state(self):
-        r"""Return the instance of :class:`~graph_tool.inference.partition_modes.PartitionModeState`
+        r"""Return the instance of :class:`~graph_tool.inference.PartitionModeState`
         representing the model at the upper hierarchical level.
         """
         base = self._base.get_coupled_state()
@@ -255,6 +261,7 @@ class PartitionModeState(object):
         otherwise using posterior mean estimates."""
         return self._base.sample_nested_partition(MLE, fix_empty, _get_rng())
 
+@entropy_state_signature
 class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
     r"""The mixed random label model state for a set of labelled partitions, which
     attempts to align them inside clusters with a common group labelling.
@@ -282,8 +289,8 @@ class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
        :doi:`10.1103/PhysRevX.11.021003`, :arxiv:`2005.13977`
 
     """
-    def __init__(self, bs, b=None, B=1, nested=False, relabel=True):
-
+    def __init__(self, bs, b=None, B=1, nested=False, relabel=True, **kwargs):
+        EntropyState.__init__(self)
         self.bs = []
         self.nested = nested
 
@@ -331,7 +338,8 @@ class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
 
 
     def __getstate__(self):
-        return dict(bs=self.bs, b=self.b, nested=self.nested)
+        state = EntropyState.__getstate__(self)
+        return dict(state, bs=self.bs, b=self.b, nested=self.nested)
 
     def __setstate__(self, state):
         if not state["nested"]:
@@ -340,13 +348,13 @@ class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
 
     def get_mode(self, r):
         r"""Return the mode in cluster ``r`` as an instance of
-        :class:`~graph_tool.inference.partition_modes.PartitionModeState`. """
+        :class:`~graph_tool.inference.PartitionModeState`. """
         base = self._state.get_mode(r);
         return PartitionModeState(None, base=base, nested=self.nested)
 
     def get_modes(self, sort=True):
         r"""Return the list of nonempty modes, as instances of
-        :class:`~graph_tool.inference.partition_modes.PartitionModeState`. If `sorted == True`,
+        :class:`~graph_tool.inference.PartitionModeState`. If `sorted == True`,
         the modes are retured in decreasing order with respect to their size.
         """
         modes = [self.get_mode(r) for r in np.unique(self.b)]
@@ -417,11 +425,11 @@ class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
         return self._state.relabel_modes(epsilon, maxiter)
 
     @copy_state_wrap
-    def entropy(self):
-        r"""Return the model entropy (negative log-likelihood)."""
+    def _entropy(self):
+        r"""Return the description length (negative joint log-likelihood)."""
         return self._state.entropy()
 
-    def _get_entropy_args(self, kwargs):
+    def _gen_eargs(self, eargs):
         return ""
 
     def posterior_entropy(self, MLE=True):
@@ -473,15 +481,52 @@ class ModeClusterState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
                                                                   self._state,
                                                                   _get_rng())
 
+def _get_array(x):
+    if isinstance(x, PropertyMap):
+        return x.fa
+    return x
+
+def _handle_props(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        nargs = (_get_array(x) for x in args)
+        ret = func(*nargs, **kwargs)
+        if isinstance(ret, np.ndarray) and isinstance(args[0], PropertyMap):
+            nret = args[0].copy()
+            nret.fa = ret
+            return nret
+        return ret
+    return wrapper
+
+def _handle_nprops(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        nargs = ([_get_array(xl) for xl in x] for x in args)
+        ret = func(*nargs, **kwargs)
+        if isinstance(ret, list):
+            nret = []
+            for i in range(len(ret)):
+                val = ret[i]
+                if isinstance(val, np.ndarray) and isinstance(args[0][i], PropertyMap):
+                    nval = args[0][i].copy()
+                    nval.fa = val
+                    nret.append(nval)
+                else:
+                    nret.append(val)
+            return nret
+        return ret
+    return wrapper
+
+@_handle_props
 def partition_overlap(x, y, norm=True):
     r"""Returns the maximum overlap between partitions, according to an optimal
     label alignment.
 
     Parameters
     ----------
-    x : iterable of ``int`` values
+    x : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         First partition.
-    y : iterable of ``int`` values
+    y : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         Second partition.
     norm : (optional, default: ``True``)
         If ``True``, the result will be normalized in the range :math:`[0,1]`.
@@ -523,7 +568,7 @@ def partition_overlap(x, y, norm=True):
     --------
     >>> x = np.random.randint(0, 10, 1000)
     >>> y = np.random.randint(0, 10, 1000)
-    >>> gt.partition_overlap(x, y)
+    >>> print(gt.partition_overlap(x, y))
     0.143
 
     References
@@ -542,20 +587,23 @@ def partition_overlap(x, y, norm=True):
     """
     x = np.asarray(x, dtype="int32")
     y = np.asarray(y, dtype="int32")
+    if len(x) != len(y):
+        raise ValueError("Partitions need to have the same number of elements")
     m = libinference.partition_overlap(x, y)
     if norm:
-        m /= max(len(x), len(y))
+        m /= max((len(x), len(y)))
     return m
 
+@_handle_nprops
 def nested_partition_overlap(x, y, norm=True):
     r"""Returns the hierarchical maximum overlap between nested partitions, according
     to an optimal recursive label alignment.
 
     Parameters
     ----------
-    x : iterable of iterables of ``int`` values
+    x : iterable of iterables of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
         First partition.
-    y : iterable of iterables of ``int`` values
+    y : iterable of iterables of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
         Second partition.
     norm : (optional, default: ``True``)
         If ``True``, the result will be normalized in the range :math:`[0,1]`.
@@ -603,8 +651,8 @@ def nested_partition_overlap(x, y, norm=True):
 
     >>> x = [np.random.randint(0, 100, 1000), np.random.randint(0, 10, 100), np.random.randint(0, 3, 10)]
     >>> y = [np.random.randint(0, 100, 1000), np.random.randint(0, 10, 100), np.random.randint(0, 3, 10)]
-    >>> gt.nested_partition_overlap(x, y)
-    0.150858...
+    >>> print(gt.nested_partition_overlap(x, y))
+    0.140018...
 
 
     References
@@ -623,30 +671,31 @@ def nested_partition_overlap(x, y, norm=True):
     """
     y = order_nested_partition_labels(y)
     x = align_nested_partition_labels(x, y)
-    L = min(len(x), len(y))
+    L = min((len(x), len(y)))
     m = 0
     N = 0
     for l in range(L):
         xl = x[l]
         yl = y[l]
-        Nl = min(len(xl), len(yl))
+        Nl = min((len(xl), len(yl)))
         null = np.logical_and(xl[:Nl] == -1, y[:Nl] == -1).sum()
         ml = (xl[:Nl] == yl[:Nl]).sum() - null
-        Nl = max((xl != -1).sum(), (yl != -1).sum())
+        Nl = max(((xl != -1).sum(), (yl != -1).sum()))
         N += Nl
         m += ml
     if norm:
         return 1 - (N - m) / (N - L)
     return m
 
+@_handle_props
 def contingency_graph(x, y):
     r"""Returns the contingency graph between both partitions.
 
     Parameters
     ----------
-    x : iterable of ``int`` values
+    x : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         First partition.
-    y : iterable of ``int`` values
+    y : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         Second partition.
 
     Returns
@@ -677,13 +726,13 @@ def contingency_graph(x, y):
     >>> y = np.random.randint(0, 10, 1000)
     >>> g = gt.contingency_graph(x, y)
     >>> g.ep.mrs.a
-    PropertyArray([ 8,  6,  8, 15, 15, 14, 11, 13,  8,  9, 16,  6,  5, 11,  8,
-                   15,  6,  8,  9, 12, 11,  8, 13,  6, 10, 14, 12, 14, 15, 18,
-                   13, 15, 10, 12, 13,  6, 12, 13, 15,  9, 11, 11,  5,  7, 11,
-                    6,  8, 15, 15, 14,  8,  8,  7, 13, 11, 11,  8, 11,  9, 11,
-                    9, 16, 13, 12,  8, 16,  6, 10, 15, 14,  4,  4,  7, 12, 11,
-                    8,  6, 16, 11, 13,  3,  5, 13,  9, 11,  4,  4, 12,  7,  5,
-                    7, 10,  6,  8,  6,  7, 10,  7, 11,  2], dtype=int32)
+    PropertyArray([ 8, 10,  4, 11,  8, 11, 15, 12, 14, 11,  8, 12,  9,  9, 11,
+                   14, 10,  9, 16,  8, 12, 16, 15, 13, 12,  7, 11, 13, 18,  9,
+                    9,  9, 14, 10, 11,  8,  6,  7, 11, 11,  8, 11, 14, 12,  8,
+                    7,  7,  8,  8, 12, 10, 11,  8, 15,  6, 13, 14, 14,  8, 10,
+                    8, 11,  7,  6, 10, 13, 10, 13,  6, 11, 15,  5,  4, 10, 13,
+                    8,  8,  9, 14,  8,  6, 11,  7,  8, 15, 10,  9,  7,  8, 11,
+                   10, 11,  9,  8,  8, 10,  7,  8,  3,  9], dtype=int32)
     """
     x = np.asarray(x, dtype="int32")
     y = np.asarray(y, dtype="int32")
@@ -698,17 +747,18 @@ def contingency_graph(x, y):
                                        x, y)
     return g
 
+@_handle_props
 def shuffle_partition_labels(x):
     r"""Returns a copy of partition ``x``, with the group labels randomly shuffled.
 
     Parameters
     ----------
-    x : iterable of ``int`` values
+    x : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         Partition.
 
     Returns
     -------
-    y : :class:`numpy.ndarray`
+    y : :class:`numpy.ndarray` or :class:`~graph_tool.PropertyMap`
         Partition with shuffled labels.
 
     Examples
@@ -722,42 +772,44 @@ def shuffle_partition_labels(x):
     libinference.partition_shuffle_labels(x, _get_rng())
     return x
 
+@_handle_nprops
 def shuffle_nested_partition_labels(x):
     r"""Returns a copy of nested partition ``x``, with the group labels randomly shuffled.
 
     Parameters
     ----------
-    x : iterable iterable of ``int`` values
+    x : iterable iterable of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
         Partition.
 
     Returns
     -------
-    y : list of :class:`numpy.ndarray`
+    y : list of :class:`numpy.ndarray` or list of :class:`~graph_tool.PropertyMap`
         Nested partition with shuffled labels.
 
     Examples
     --------
     >>> x = [[0, 0, 0, 1, 1, 1, 2, 2, 2], [0, 0, 1], [1, 0]]
     >>> gt.shuffle_nested_partition_labels(x)
-    [array([1, 1, 1, 0, 0, 0, 2, 2, 2], dtype=int32), array([0, 0, 1], dtype=int32), array([0, 1], dtype=int32)]
+    [array([0, 0, 0, 2, 2, 2, 1, 1, 1], dtype=int32), array([1, 0, 1], dtype=int32), array([1, 0], dtype=int32)]
     """
 
     x = [np.asarray(xl, dtype="int32") for xl in x]
     x = libinference.nested_partition_shuffle_labels(x, _get_rng())
     return x
 
+@_handle_props
 def order_partition_labels(x):
     r"""Returns a copy of partition ``x``, with the group labels ordered decreasingly
     according to group size.
 
     Parameters
     ----------
-    x : iterable of ``int`` values
+    x : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         Partition.
 
     Returns
     -------
-    y : :class:`numpy.ndarray`
+    y : :class:`numpy.ndarray` or :class:`~graph_tool.PropertyMap`
         Partition with ordered labels.
 
     Examples
@@ -771,18 +823,19 @@ def order_partition_labels(x):
     libinference.partition_order_labels(x)
     return x
 
+@_handle_nprops
 def order_nested_partition_labels(x):
     r"""Returns a copy of nested partition ``x``, with the group labels ordered
     decreasingly according to group size at each level.
 
     Parameters
     ----------
-    x : iterable of iterables of ``int`` values
+    x : iterable of iterables of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
         Partition.
 
     Returns
     -------
-    y : list of :class:`numpy.ndarray`
+    y : list of :class:`numpy.ndarray` or list of :class:`~graph_tool.PropertyMap`
         Nested partition with ordered labels.
 
     Examples
@@ -796,19 +849,22 @@ def order_nested_partition_labels(x):
     x = libinference.nested_partition_order_labels(x)
     return x
 
+@_handle_props
 def align_partition_labels(x, y):
     r"""Returns a copy of partition ``x``, with the group labels aligned as to
     maximize the overlap with ``y``.
 
     Parameters
     ----------
-    x : iterable of ``int`` values
+    x : iterable of ``int`` values or :class:`~graph_tool.PropertyMap`
         Partition.
+    y : iterables of ``int`` values or :class:`~graph_tool.PropertyMap`
+        Reference partition.
 
     Returns
     -------
-    y : :class:`numpy.ndarray`
-        Partition with aligned labels.
+    z : :class:`numpy.ndarray` or :class:`~graph_tool.PropertyMap`
+        Partition ``x`` with labels aligned to ``y``.
 
     Notes
     -----
@@ -823,7 +879,7 @@ def align_partition_labels(x, y):
     >>> x = [0, 2, 2, 1, 1, 1, 2, 3, 2]
     >>> y = gt.shuffle_partition_labels(x)
     >>> print(y)
-    [3 0 0 1 1 1 0 2 0]
+    [3 1 1 2 2 2 1 0 1]
     >>> gt.align_partition_labels(y, x)
     array([0, 2, 2, 1, 1, 1, 2, 3, 2], dtype=int32)
 
@@ -839,19 +895,22 @@ def align_partition_labels(x, y):
     libinference.align_partition_labels(x, y)
     return x
 
+@_handle_nprops
 def align_nested_partition_labels(x, y):
     r"""Returns a copy of nested partition ``x``, with the group labels aligned as to
     maximize the overlap with ``y``.
 
     Parameters
     ----------
-    x : iterable of iterables of ``int`` values
-        Partition.
+    x : iterable of iterables of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
+        Nested partition.
+    y : iterable of iterables of ``int`` values or iterable of :class:`~graph_tool.PropertyMap`
+        Reference nested partition.
 
     Returns
     -------
-    y : list of :class:`numpy.ndarray`
-        Nested partition with aligned labels.
+    z : list of :class:`numpy.ndarray` or list of :class:`~graph_tool.PropertyMap`
+        Nested partition ``x`` with labels aligned to ``y``.
 
     Notes
     -----
@@ -866,7 +925,7 @@ def align_nested_partition_labels(x, y):
     >>> x = [[0, 2, 2, 1, 1, 1, 2, 3, 2], [1, 0, 1, 0], [0,0]]
     >>> y = gt.shuffle_nested_partition_labels(x)
     >>> print(y)
-    [array([1, 3, 3, 2, 2, 2, 3, 0, 3], dtype=int32), array([1, 0, 1, 0], dtype=int32), array([0, 0], dtype=int32)]
+    [array([3, 1, 1, 2, 2, 2, 1, 0, 1], dtype=int32), array([0, 1, 0, 1], dtype=int32), array([0, 0], dtype=int32)]
     >>> gt.align_nested_partition_labels(y, x)
     [array([0, 2, 2, 1, 1, 1, 2, 3, 2], dtype=int32), array([1, 0, 1, 0], dtype=int32), array([0, 0], dtype=int32)]
     """
@@ -876,13 +935,14 @@ def align_nested_partition_labels(x, y):
     x = libinference.align_nested_partition_labels(x, y)
     return x
 
+@_parallel
 def partition_overlap_center(bs, init=None, relabel_bs=False):
     r"""Find a partition with a maximal overlap to all items of the list of
     partitions given.
 
     Parameters
     ----------
-    bs : list of iterables of ``int`` values
+    bs : list of iterables of ``int`` values or list of :class:`~graph_tool.PropertyMap`
         List of partitions.
     init : iterable of ``int`` values (optional, default: ``None``)
         If given, it will determine the initial partition.
@@ -927,7 +987,7 @@ def partition_overlap_center(bs, init=None, relabel_bs=False):
     number of partitions, :math:`N` is the length of the partitions and
     :math:`B` is the number of labels used.
 
-    If enabled during compilation, this algorithm runs in parallel.
+    @parallel@
 
     Examples
     --------
@@ -938,10 +998,10 @@ def partition_overlap_center(bs, init=None, relabel_bs=False):
     ...     y[np.random.randint(len(y))] = np.random.randint(5)
     ...     bs.append(y)
     >>> bs[:3]
-    [array([5, 5, 2, 0, 1, 2, 1, 0, 0, 0, 0]), array([1, 5, 2, 0, 1, 0, 1, 0, 0, 0, 0]), array([5, 5, 2, 0, 1, 0, 1, 0, 4, 0, 0])]
+    [array([5, 5, 2, 0, 1, 0, 1, 0, 0, 0, 3]), array([5, 5, 2, 0, 4, 0, 1, 0, 0, 0, 0]), array([5, 5, 1, 0, 1, 0, 1, 0, 0, 0, 0])]
     >>> c, r = gt.partition_overlap_center(bs)
     >>> print(c, r)
-    [1 1 2 0 3 0 3 0 0 0 0] 0.07454545...
+    [1 1 2 0 3 0 3 0 0 0 0] 0.06818181...
     >>> gt.align_partition_labels(c, x)
     array([5, 5, 2, 0, 1, 0, 1, 0, 0, 0, 0], dtype=int32)
 
@@ -952,9 +1012,8 @@ def partition_overlap_center(bs, init=None, relabel_bs=False):
        :doi:`10.1103/PhysRevX.11.021003`, :arxiv:`2005.13977`
     """
 
-    if relabel_bs:
-        bs = np.asarray(bs, dtype="int32")
-    else:
+    bs = _get_array(bs)
+    if not relabel_bs:
         bs = np.array(bs, dtype="int32")
     if init is None:
         c = np.zeros(len(bs[0]), dtype="int32")
@@ -964,13 +1023,14 @@ def partition_overlap_center(bs, init=None, relabel_bs=False):
     r = libinference.partition_overlap_center(bs, c)
     return c, r
 
+@_parallel
 def nested_partition_overlap_center(bs, init=None, return_bs=False):
     r"""Find a nested partition with a maximal overlap to all items of the list of
     nested partitions given.
 
     Parameters
     ----------
-    bs : list of list of iterables of ``int`` values
+    bs : list of list of iterables of ``int`` values or list of list of :class:`~graph_tool.PropertyMap`
         List of nested partitions.
     init : iterable of iterables of ``int`` values (optional, default: ``None``)
         If given, it will determine the initial nested partition.
@@ -1017,7 +1077,7 @@ def nested_partition_overlap_center(bs, init=None, return_bs=False):
     is the number of partitions, :math:`N_l` is the length of the partitions and
     :math:`B_l` is the number of labels used, in level :math:`l`.
 
-    If enabled during compilation, this algorithm runs in parallel.
+    @parallel@
 
     Examples
     --------
@@ -1029,10 +1089,10 @@ def nested_partition_overlap_center(bs, init=None, return_bs=False):
     ...     y[1][np.random.randint(len(y[1]))] = np.random.randint(2)
     ...     bs.append(y)
     >>> bs[:3]
-    [[array([5, 5, 2, 0, 1, 0, 3, 0, 0, 0, 0]), array([0, 1, 1, 1, 1, 1])], [array([5, 5, 2, 0, 0, 0, 1, 0, 0, 0, 0]), array([0, 0, 0, 1, 1, 1])], [array([1, 5, 2, 0, 1, 0, 1, 0, 0, 0, 0]), array([0, 1, 0, 1, 1, 1])]]
+    [[array([5, 5, 2, 0, 1, 0, 1, 0, 0, 0, 3]), array([0, 1, 0, 1, 1, 1])], [array([5, 5, 2, 0, 1, 0, 4, 0, 0, 0, 0]), array([0, 1, 1, 1, 1, 1])], [array([5, 5, 2, 0, 1, 0, 1, 0, 0, 0, 2]), array([0, 1, 0, 1, 1, 1])]]
     >>> c, r = gt.nested_partition_overlap_center(bs)
     >>> print(c, r)
-    [array([1, 1, 2, 0, 3, 0, 3, 0, 0, 0, 0], dtype=int32), array([0, 1, 0, 1, 1], dtype=int32)] 0.084492...
+    [array([1, 1, 2, 0, 3, 0, 3, 0, 0, 0, 0], dtype=int32), array([0, 1, 0, 1, 1], dtype=int32)] 0.069385...
     >>> gt.align_nested_partition_labels(c, x)
     [array([5, 5, 2, 0, 1, 0, 1, 0, 0, 0, 0], dtype=int32), array([ 0,  1,  0, -1, -1,  1], dtype=int32)]
 
@@ -1043,7 +1103,7 @@ def nested_partition_overlap_center(bs, init=None, return_bs=False):
        :doi:`10.1103/PhysRevX.11.021003`, :arxiv:`2005.13977`
     """
 
-    bs = [[np.asarray(bs[m][l], dtype="int32") for l in range(len(bs[m]))] for m in range(len(bs))]
+    bs = [[np.asarray(_get_array(bs[m][l]), dtype="int32") for l in range(len(bs[m]))] for m in range(len(bs))]
     if init is None:
         c = [np.zeros(len(bs[0][l]), dtype="int32") for l in range(len(bs[0]))]
     else:
@@ -1054,13 +1114,14 @@ def nested_partition_overlap_center(bs, init=None, return_bs=False):
     else:
         return c, r
 
+@_handle_nprops
 def nested_partition_clear_null(x):
     r"""Returns a copy of nested partition ``x`` where the null values ``-1`` are
     replaced with ``0``.
 
     Parameters
     ----------
-    x : iterable of iterables of ``int`` values
+    x : iterable of iterables of ``int`` values or list of :class:`~graph_tool.PropertyMap`
         Partition.
 
     Returns
@@ -1071,7 +1132,7 @@ def nested_partition_clear_null(x):
     Notes
     -----
     This is useful to pass hierarchical partitions to
-    :class:`~graph_tool.inference.nested_blockmodel.NestedBlockState`.
+    :class:`~graph_tool.inference.NestedBlockState`.
 
     Examples
     --------
