@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2024 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -19,8 +19,8 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """
-``graph_tool.search`` - Search algorithms
------------------------------------------
+``graph_tool.search``
+---------------------
 
 This module includes several search algorithms, which are customizable to
 arbitrary purposes. It is mostly a wrapper around the Visitor interface of the
@@ -33,6 +33,8 @@ Summary
 
 .. autosummary::
    :nosignatures:
+   :toctree: autosummary
+   :template: class.rst
 
    bfs_search
    bfs_iterator
@@ -50,8 +52,8 @@ Summary
    AStarVisitor
    StopSearch
 
-Examples
-++++++++
+Example network
++++++++++++++++
 
 In this module, most documentation examples will make use of the network
 :download:`search_example.xml <search_example.xml>`, shown below.
@@ -72,9 +74,6 @@ In this module, most documentation examples will make use of the network
    This is the network used in the examples below. The width of the edges
    correspond to the values of the "weight" property map.
 
-
-Contents
-++++++++
 """
 
 from .. dl_import import dl_import
@@ -235,6 +234,8 @@ def bfs_search(g, source=None, visitor=BFSVisitor()):
 
     With the above class defined, we can perform the BFS search as follows.
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
     >>> dist = g.new_vertex_property("int")
     >>> pred = g.new_vertex_property("int64_t")
     >>> gt.bfs_search(g, g.vertex(0), VisitorExample(name, pred, dist))
@@ -323,6 +324,8 @@ def bfs_iterator(g, source=None, array=False):
     Examples
     --------
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
     >>> for e in gt.bfs_iterator(g, g.vertex(0)):
     ...    print(name[e.source()], "->", name[e.target()])
     Bob -> Eve
@@ -517,6 +520,8 @@ def dfs_search(g, source=None, visitor=DFSVisitor()):
 
     With the above class defined, we can perform the DFS search as follows.
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
     >>> time = g.new_vertex_property("int")
     >>> pred = g.new_vertex_property("int64_t")
     >>> gt.dfs_search(g, g.vertex(0), VisitorExample(name, pred, time))
@@ -627,6 +632,8 @@ def dfs_iterator(g, source=None, array=False):
     Examples
     --------
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
     >>> for e in gt.dfs_iterator(g, g.vertex(0)):
     ...    print(name[e.source()], "->", name[e.target()])
     Bob -> Eve
@@ -696,7 +703,7 @@ class DijkstraVisitor(object):
         ::
 
             (u, v) = tuple(e)
-            assert(compare(combine(d[u], weight[e]), d[v]))
+            assert compare(combine(d[u], weight[e]), d[v])
 
         """
         return
@@ -848,6 +855,9 @@ def dijkstra_search(g, weight, source=None, visitor=DijkstraVisitor(), dist_map=
 
     With the above class defined, we can perform the Dijkstra search as follows.
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
+    >>> weight = g.ep["weight"]
     >>> time = g.new_vertex_property("int")
     >>> dist, pred = gt.dijkstra_search(g, weight, g.vertex(0), VisitorExample(name, time))
     --> Bob has been discovered!
@@ -1018,6 +1028,9 @@ def dijkstra_iterator(g, weight, source=None, dist_map=None, combine=None,
     Examples
     --------
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
+    >>> weight = g.ep["weight"]
     >>> for e in gt.dijkstra_iterator(g, weight, g.vertex(0)):
     ...    print(name[e.source()], "->", name[e.target()])
     Bob -> Eve
@@ -1284,6 +1297,9 @@ def bellman_ford_search(g, source, weight, visitor=BellmanFordVisitor(),
     With the above class defined, we can perform the Bellman-Ford search as
     follows.
 
+    >>> g = gt.load_graph("search_example.xml")
+    >>> name = g.vp["name"]
+    >>> weight = g.ep["weight"]
     >>> nweight = g.copy_property(weight)
     >>> nweight.a[6] *= -1   # include negative weight in edge (Carlos, Alice)
     >>> minimized, dist, pred = gt.bellman_ford_search(g, g.vertex(0), nweight, VisitorExample(name))
@@ -1400,7 +1416,7 @@ class AStarVisitor(object):
         ::
 
             (u, v) = tuple(e)
-            assert(compare(combine(d[u], weight[e]), d[v]))
+            assert compare(combine(d[u], weight[e]), d[v])
 
         """
         return
@@ -1596,6 +1612,7 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
         import matplotlib.cm
         seed(42)
 
+
     >>> points = random((500, 2)) * 4
     >>> points[0] = [-0.01, 0.01]
     >>> points[1] = [4.01, 4.01]
